<?php
/**
 * Simple Gallery browse action
 *
 * $Id: Browse.php 203 2011-02-18 17:46:16Z indy $
 *
 * $LastChangedBy: indy $
 *
 * $LastChangedDate: 2011-02-18 19:46:16 +0200 (Fri, 18 Feb 2011) $
 *
 * $Rev: 203 $
 *
 * @author Indiana Jones <indy2kro@yahoo.com>
 * @package SimpleGallery
 * @version 1.0
 * @copyright 2011 Simple Gallery
 */

/**
 * Simple Gallery browse action
 */
class SimpleGallery_Action_Browse extends SimpleGallery_Action_Abstract
{
    /**
     * Source directory path
     * 
     * @var string
     */
    protected $_sourcePath;

    /**
     * Page title
     * 
     * @var string
     */
    protected $_title;

    /**
     * Full path to browse
     * 
     * @var string
     */
    protected $_fullPath;

    /**
     * Directories in the current path
     *
     * @var array
     */
    protected $_directories = array();

    /**
     * File in the current path
     *
     * @var array
     */
    protected $_files = array();

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        $config = SimpleGallery_Registry::get('config');

        $this->_sourcePath = $config['source_directory'];
    }

    /**
     * Main run method
     *
     * @param string $path
     * @throws SimpleGallery_Exception
     * @return void
     */
    public function run(Array $params = array())
    {
        parent::run($params);

        // build full path
        $this->_buildFullPath();

        if (!is_dir($this->_fullPath)) {
            throw new SimpleGallery_Exception('Provided path does not exist.');
        }

        // check if the given path is valid
        $this->_checkPath();

        // get files list
        $this->_getFilesList();

        // display page header
        $this->_displayHeader();

        // display page content
        $this->_displayContent();

        // display page footer
        $this->_displayFooter();
    }

    /**
     * Display header
     *
     * @return void
     */
    protected function _displayHeader()
    {
        $config = SimpleGallery_Registry::get('config');

        $this->_buildTitle();

        $pageTitle = '';

        if (!empty($this->_title)) {
            $pageTitle = $this->_title;

            if (!empty($config['gallery_name'])) {
                $pageTitle .= ' - ' . $config['gallery_name'];
            }
        } else {
            $pageTitle = $config['gallery_name'];
        }

        echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>' . htmlspecialchars($pageTitle) . '</title>
<link rel="stylesheet" type="text/css" href="' . $config['public_url'] . '/style.css" />
</head>

<body>
<div id="simplegallery-container">
';
        $this->_displayTitle();
    }

    /**
     * Display content
     *
     * @return void
     */
    protected function _displayContent()
    {
        $config = SimpleGallery_Registry::get('config');

        if (!empty($this->_directories)) {
            echo '<ul class="simplegallery-dirs">' . PHP_EOL;

            foreach ($this->_directories as $directory) {
                echo '<li class="simplegallery-dir">';
                echo '<a href="' . $directory['path'] . '/">' . htmlspecialchars($directory['name']) . '</a>';

                if ($config['count_images']) {
                    echo ' <small>(' . $directory['count_images'] . ' images)</small>';
                }
                
                echo '</li>' . PHP_EOL;
            }
            
            echo '</ul>' . PHP_EOL;
        }

        foreach ($this->_files as $file) {
            echo '<div class="simplegallery-img">';
            echo '<a href="' . $file['path'] . '"><img src="' . $file['thumb'] . '" alt="' . htmlspecialchars($file['name'])
                . '" title="' . htmlspecialchars($file['title']) . '" /></a>' . PHP_EOL;
            echo '</div>' . PHP_EOL;
        }
    }

    /**
     * Display footer
     *
     * @return void
     */
    protected function _displayFooter()
    {
        $config = SimpleGallery_Registry::get('config');
        
        if ('/' != $this->_path) {
            echo '<div class="simplegallery-backtoindex"><a href="' . $config['public_url'] . '/">Back to index</a></div>' . PHP_EOL;
        }

        parent::_displayFooter();
    }

    /**
     * Display page title
     * 
     * @return void
     */
    protected function _displayTitle()
    {
        $config = SimpleGallery_Registry::get('config');

        $title = $this->_title;

        if ('/' == $this->_path) {
            $title = $config['gallery_name'];
        }
        
        echo '<h1>' . htmlspecialchars($title) . '</h1>';
    }

    /**
     * Build page title
     *
     * @return void
     */
    protected function _buildTitle()
    {
        $title = trim($this->_path, '/');

        $this->_title = $title;
    }
    
    /**
     * Build full path
     *
     * @return void
     */
    protected function _buildFullPath()
    {
        $fullPath = $this->_cleanDirectoryPath($this->_sourcePath) . $this->_cleanDirectoryPath($this->_path);

        $this->_fullPath = $this->_cleanDirectoryPath($fullPath);
    }

    /**
     * Get files list
     *
     * @return array
     */
    protected function _getFilesList()
    {
        $config = SimpleGallery_Registry::get('config');

        $fileExtensions = explode(',', $config['file_extensions']);
        $fileExtensions = implode('|', $fileExtensions);

        $acceptedPattern = '/(' . $fileExtensions . ')$/i';

        $dh = opendir($this->_fullPath);

        if (!$dh) {
            throw new SimpleGallery_Exception('Failed to open directory.');
        }

        while (($file = readdir($dh)) !== false) {
            // skip files that start with .
            if (preg_match('/^\./', $file)) {
                continue;
            }

            $fullPath = $this->_fullPath . $file;

            $filePath = $config['public_url'] . '/';

            if ('/' != $this->_path) {
                $filePath .= ltrim($this->_path, '/');
            }

            $thumbPublicPath = $filePath . $config['thumbnail_directory'] . '/' . $file;

            $filePath .= $file;

            $fileItem = array(
                'name' => $file,
                'path' => $filePath,
                'title' => $this->_getTitle($file)
            );

            if (is_dir($fullPath)) {
                if ($file == $config['thumbnail_directory']) {
                    continue;
                }

                if ($config['count_images']) {
                    $fileItem['count_images'] = $this->_countImages($this->_path . $file . '/');
                }
                
                $this->_directories[] = $fileItem;
            } else {
                if (!preg_match($acceptedPattern, $file)) {
                    continue;
                }

                $thumbPath = $this->_fullPath . $config['thumbnail_directory'] . '/' . $file;
                
                $fileItem['thumb'] = $thumbPublicPath;
                $fileItem['path'] .= '.html';

                $this->_files[] = $fileItem;
            }
        }

        closedir($dh);
    }

    /**
     * Check if the given path is valid
     *
     * @throws SimpleGallery_Exception
     * @return void
     */
    protected function _checkPath()
    {
        $folderName = basename($this->_path);

        $config = SimpleGallery_Registry::get('config');

        // don't allow access to thumbnail directories, this can cause an infinite loop
        if ($folderName == $config['thumbnail_directory']) {
            throw new SimpleGallery_Exception('Access denied to this folder.');
        }
    }

    /**
     * Count images in a gallery
     * 
     * @param string $path The gallery path
     * @return integer
     */
    protected function _countImages($path)
    {
        $dir = new SimpleGallery_Directory();
        $countImages = $dir->countImages($path, $this->_path);

        return $countImages;
    }
}

/* EOF */