<?php
/**
 * Simple Gallery view image action
 *
 * $Id: View.php 203 2011-02-18 17:46:16Z indy $
 *
 * $LastChangedBy: indy $
 *
 * $LastChangedDate: 2011-02-18 19:46:16 +0200 (Fri, 18 Feb 2011) $
 *
 * $Rev: 203 $
 *
 * @author Indiana Jones <indy2kro@yahoo.com>
 * @package SimpleGallery
 * @version 1.0
 * @copyright 2011 Simple Gallery
 */

/**
 * Simple Gallery view image action
 */
class SimpleGallery_Action_View extends SimpleGallery_Action_Abstract
{
    /**
     * Page title
     *
     * @var string
     */
    protected $_title;

    /**
     * Gallery name
     *
     * @var string
     */
    protected $_gallery;

    /**
     * Navigation images
     *
     * @var array
     */
    protected $_navImages = array();

    /**
     * Current file name
     * 
     * @var string
     */
    protected $_fileName;

    /**
     * Main run method
     * 
     * @param array $params Action parameters
     * @throws SimpleGallery_Exception
     * @return void
     */
    public function run(Array $params = array())
    {
        parent::run($params);

        $this->_path = rtrim($this->_path, '/');

        if (!file_exists($this->_path)) {
            throw new SimpleGallery_Exception('Required image does not exist.');
        }

        $this->_fileName = basename($this->_path);

        // resize image
        $this->_resizeImage();

        // display page header
        $this->_displayHeader();

        // display page content
        $this->_displayContent();

        // display page footer
        $this->_displayFooter();
    }

    /**
     * Resize image based on max config values
     *
     * @throws SimpleGallery_Exception
     * @return void
     */
    protected function _resizeImage()
    {
        $config = SimpleGallery_Registry::get('config');

        if (!$config['auto_resize']) {
            return null;
        }

        $options = array(
            SimpleGallery_Image::OPTION_HEIGHT => $config['max_height'],
            SimpleGallery_Image::OPTION_WIDTH => $config['max_width']
        );

        $fullPath = $config['source_directory'] . '/' . $this->_path;
        $tmpPath = $fullPath . rand(1, 10000) . '.tmp';

        try {
            $image = new SimpleGallery_Image();
            $resizeResult = $image->resize($fullPath, $tmpPath, $options);

            if (is_null($resizeResult)) {
                // nothing to do
                return null;
            }

            if (!$resizeResult) {
                throw new SimpleGallery_Exception('Failed to resize image.');
            }

            $unlinkResult = unlink($fullPath);

            if (!$unlinkResult) {
                @unlink($tmpPath);
                throw new SimpleGallery_Exception('Failed to remove old image.');
            }
            
            $renameResult = rename($tmpPath, $fullPath);

            if (!$renameResult) {
                throw new SimpleGallery_Exception('Failed to rename temporary image.');
            }
        } catch (SimpleGallery_Exception $ex) {
            throw new SimpleGallery_Exception('Failed to create thumbnail for ' . $filePath . ' - ' . $ex->getMessage());
        }
    }

    /**
     * Display header
     *
     * @return void
     */
    protected function _displayHeader()
    {
        $config = SimpleGallery_Registry::get('config');

        $this->_buildTitle();
        $this->_buildGalleryName();

        $title = $this->_getTitle($this->_title);

        $pageTitle = $title . ' - ' . $this->_gallery;
        
        echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>' . htmlspecialchars($pageTitle) . '</title>
<link rel="stylesheet" type="text/css" href="' . $config['public_url'] . '/style.css" />
</head>

<body>
<div id="simplegallery-container">
';
        $this->_displayTitle();
    }

    /**
     * Display content
     *
     * @return void
     */
    protected function _displayContent()
    {
        $this->_displayNavigation();
        $this->_displayImage();

        echo '<div class="simplegallery-clear"></div>' . PHP_EOL;
    }

    /**
     * Display navigation bar
     *
     * @return void
     */
    protected function _displayNavigation()
    {
        $config = SimpleGallery_Registry::get('config');

        $galleryPath = dirname($this->_path) . '/';

        $directory = new SimpleGallery_Directory();
        $this->_navImages = $directory->getImages($galleryPath);

        $imgPosition = $this->_findCurrentPosition();
        $lastIndex = count($this->_navImages) - 1;

        $maxThumbs = $config['max_nav_images_around'];
        if ($imgPosition < $maxThumbs) {
            $maxThumbs += ($maxThumbs - $imgPosition);
        } elseif ($lastIndex - $imgPosition < $maxThumbs) {
            $maxThumbs += ($maxThumbs - ($lastIndex - $imgPosition));
        }

        echo '<div id="simplegallery-nav">' . PHP_EOL;

        foreach ($this->_navImages as $index => $image) {
            if (abs($index - $imgPosition) > $maxThumbs) {
                // only display a number of images around the current one
                continue;
            }

            if ($index == $imgPosition) {
                echo '<div class="simplegallery-img simplegallery-nav-img-sel">' . PHP_EOL;
                echo '<img src="' . $image['thumbnail'] . '" alt="' . htmlspecialchars($image['name']) . '" title="' . htmlspecialchars($image['title']) . '" />';
                echo '</div>' . PHP_EOL;
            } else {
                echo '<div class="simplegallery-img simplegallery-nav-img">' . PHP_EOL;
                echo '<a href="' . $image['path'] . '.html">';
                echo '<img src="' . $image['thumbnail'] . '" alt="' . htmlspecialchars($image['name']) . '" title="' . htmlspecialchars($image['title']) . '" />';
                echo '</a>';
                echo '</div>' . PHP_EOL;

            }
        }

        echo '</div>' . PHP_EOL;
    }

    /**
     * Display image
     *
     * @return void
     */
    protected function _displayImage()
    {
        $config = SimpleGallery_Registry::get('config');

        echo '<div id="simplegallery-img-large">' . PHP_EOL;

        $imgPath = $config['public_url'] . '/' . $this->_path;

        $title = $this->_getTitle($this->_title);

        echo '<img src="' . $imgPath . '" alt="' . htmlspecialchars($this->_title) . '" title="' . htmlspecialchars($title) . '" />';

        echo '</div>' . PHP_EOL;
    }

    /**
     * Display footer
     *
     * @return void
     */
    protected function _displayFooter()
    {
        $config = SimpleGallery_Registry::get('config');

        $galleryPath = dirname($this->_path);

        $backUrl = $config['public_url'] . '/' . $galleryPath;

        echo '<div class="simplegallery-backtoindex"><a href="' . $backUrl . '/">Back to gallery</a></div>' . PHP_EOL;
        
        parent::_displayFooter();
    }

    /**
     * Display page title
     *
     * @return void
     */
    protected function _displayTitle()
    {
        $config = SimpleGallery_Registry::get('config');

        $title = $this->_title;

        echo '<h1 class="simplegallery-img-large-title">' . htmlspecialchars($title) . '</h1>';
    }

    /**
     * Build page title
     *
     * @return void
     */
    protected function _buildTitle()
    {
        $title = trim(basename($this->_path), '/');

        $this->_title = $title;
    }

    /**
     * Build gallery name
     *
     * @return void
     */
    protected function _buildGalleryName()
    {
        $this->_gallery = basename(dirname($this->_path));
    }

    /**
     * Find current image position
     * 
     * @return mixed Returns the item index or false if the item was not found
     */
    protected function _findCurrentPosition()
    {
        foreach($this->_navImages as $index => $image) {
            if ($this->_fileName == $image['name']) {
                return $index;
            }
        }

        return false;
    }
}

/* EOF */