<?php
/**
 * Simple Gallery thumbnail file
 *
 * $Id: Image.php 191 2011-02-16 20:53:05Z indy $
 *
 * $LastChangedBy: indy $
 *
 * $LastChangedDate: 2011-02-16 22:53:05 +0200 (Wed, 16 Feb 2011) $
 *
 * $Rev: 191 $
 *
 * @author Indiana Jones <indy2kro@yahoo.com>
 * @package SimpleGallery
 * @version 1.0
 * @copyright 2011 Simple Gallery
 */

/**
 * Simple Gallery thumbnail creator
 */
class SimpleGallery_Thumb
{
    /**
     * Accepted formats
     */

    /**
     * JPEG file format
     */
    const FORMAT_JPG = 'jpg';

    /**
     * GIF file format
     */
    const FORMAT_GIF = 'gif';

    /**
     * PNG file format
     */
    const FORMAT_PNG = 'png';
    
    /**
     * Available option keys
     */

    /**
     * Jpeg quality option key
     */
    const OPTION_JPEG_QUALITY = 'jpeg_quality';

    /**
     * Preserve ratio option key
     */
    const OPTION_PRESERVE_RATIO = 'preserve_ratio';

    /**
     * Options
     *
     * @var array
     */
    protected $_options = array(
        self::OPTION_JPEG_QUALITY => 100,
        self::OPTION_PRESERVE_RATIO => true
    );

    /**
     * Image file path
     * 
     * @var string
     */
    protected $_file;

    /**
     * Image format
     * 
     * @var string
     */
    protected $_format;

    /**
     * GD image resource
     * 
     * @var resource
     */
    protected $_resource;

    /**
     * GD new image resource
     *
     * @var resource
     */
    protected $_newResource;

    /**
     * Image width
     *
     * @var integer
     */
    protected $_width;

    /**
     * Image height
     *
     * @var integer
     */
    protected $_height;

    /**
     * Image ratio
     * 
     * @var integer
     */
    protected $_ratio;

    /**
     * Check if true color functions are available
     * 
     * @var boolean
     */
    protected $_trueColor = true;

	/**
	 * Constructor
	 *
	 * @param string $file File for which to create thumbnail
     * @param array $options Options for the thumbnail creator
     * @throws SimpleGallery_Exception
     * @return void
	 */
	public function __construct($file, $options = array())
	{
        $this->_file = $file;

        // set options
        $this->_setOptions($options);

        // detect image format
		$this->_detectFormat();

        // check if the image format is supported
        $this->_checkFormatSupport();

        // load image to internal resource
		$this->_loadImage();

        // detect image dimensions
        $this->_detectDimensions();
	}

    /**
     * Get image width
     *
     * @return integer Width value
     */
    public function getWidth()
    {
        return $this->_width;
    }

    /**
     * Get image height
     *
     * @return integer Height value
     */
    public function getHeight()
    {
        return $this->_height;
    }

    /**
     * Resizes an image to be no larger than given values
     *
     * @param integer $maxWidth Maximum image width
     * @param integer $maxHeight Maximum image height
     * @throws SimpleGallery_Exception
     * @return void
     */
    public function resize($maxWidth, $maxHeight)
    {
        if (!is_numeric($maxWidth)) {
            throw new SimpleGallery_Exception('Max width must be numeric.');
        }

        if (!is_numeric($maxHeight)) {
            throw new SimpleGallery_Exception('Max height must be numeric.');
        }

		// compute the new image size
		$imgSize = $this->_calcImageSize($maxWidth, $maxHeight);

		// create the working image
		if ($this->_trueColor) {
			$this->_newResource = imagecreatetruecolor($imgSize['width'], $imgSize['height']);
		} else {
			$this->_newResource = imagecreate($imgSize['width'], $imgSize['height']);
		}
        
        if (false === $this->_newResource) {
            throw new SimpleGallery_Exception('Failed to create GD object.');
        }

		// and create the newly sized image
		imagecopyresampled (
			$this->_newResource,
			$this->_resource,
			0,
			0,
			0,
			0,
			$imgSize['width'],
			$imgSize['height'],
			$this->_width,
			$this->_height
		);

		// update old variables
		$this->_resource = $this->_newResource;
		$this->_width = $imgSize['width'];
		$this->_height = $imgSize['height'];
    }

    /**
     * Resize an image adaptively. The resulting image will have exactly the given
     * max width and height, cropping the image in the center for it for best fit
     * after the resize is made
     *
     * @param integer $maxWidth Maximum image width
     * @param integer $maxHeight Maximum image height
     * @throws SimpleGallery_Exception
     * @return void
     */
    public function adaptiveResize($maxWidth, $maxHeight)
    {
        if (!is_numeric($maxWidth)) {
            throw new SimpleGallery_Exception('Max width must be numeric.');
        }

        if (!is_numeric($maxHeight)) {
            throw new SimpleGallery_Exception('Max height must be numeric.');
        }

		// compute the new image size
		$imgSize = $this->_calcAdaptiveImageSize($maxWidth, $maxHeight);

        $this->resize($imgSize['width'], $imgSize['height']);

		// create the working image
		if ($this->_trueColor) {
			$this->_newResource = imagecreatetruecolor($imgSize['cropWidth'], $imgSize['cropWidth']);
		} else {
			$this->_newResource = imagecreate($imgSize['cropWidth'], $imgSize['cropWidth']);
		}

        if (false === $this->_newResource) {
            throw new SimpleGallery_Exception('Failed to create GD object.');
        }

		$cropX = 0;
		$cropY = 0;

		// now, figure out how to crop the rest of the image...
		if ($this->_width > $maxWidth) {
			$cropX = intval(($this->_width - $maxWidth) / 2);
		} elseif ($this->_height > $maxHeight) {
			$cropY = intval(($this->_height - $maxHeight) / 2);
		}

		// and create the newly sized image
		imagecopyresampled (
			$this->_newResource,
			$this->_resource,
			0,
			0,
			$cropX,
			$cropY,
			$imgSize['cropWidth'],
			$imgSize['cropHeight'],
			$imgSize['cropWidth'],
			$imgSize['cropHeight']
		);

		// update old variables
		$this->_resource = $this->_newResource;
		$this->_width = $imgSize['cropWidth'];
		$this->_height = $imgSize['cropHeight'];
    }

    /**
     * Save the new image
     *
     * @param string $destination The destination where to save the image
     * @param string $format The destination file format
     * @throws SimpleGallery_Exception
     * @return void
     */
    public function save($destination, $format = null)
    {
        if (is_null($format)) {
            $format = $this->_format;
        }

        $this->_validateFormat($format);

		switch ($format) {
			case self::FORMAT_GIF:
				$saveResult = imagegif($this->_resource, $destination);
				break;

			case self::FORMAT_JPG:
				$saveResult = imagejpeg($this->_resource, $destination, $this->_options[self::OPTION_JPEG_QUALITY]);
				break;

			case self::FORMAT_PNG:
				$saveResult = imagepng($this->_resource, $destination);
				break;

            default:
                throw new SimpleGallery_Exception('Unsupported format.');
                break;
		}

        if (!$saveResult) {
            throw new SimpleGallery_Exception('Failed to save image.');
        }
    }

    /**
     * Detect image format
     *
     * @throws SimpleGallery_Exception
     * @return void
     */
    protected function _detectFormat()
    {
        if (!file_exists($this->_file)) {
            throw new SimpleGallery_Exception('Failed to open image file.');
        }

		$formatInfo = getimagesize($this->_file);

        if (false === $formatInfo) {
            throw new SimpleGallery_Exception('Image file is not in valid format.');
        }

		$mimeType = isset($formatInfo['mime']) ? $formatInfo['mime'] : null;

		switch ($mimeType)
		{
			case 'image/gif':
				$this->_format = self::FORMAT_GIF;
				break;

			case 'image/jpeg':
			case 'image/jpg':
				$this->_format = self::FORMAT_JPG;
				break;

			case 'image/png':
				$this->_format = self::FORMAT_PNG;
				break;

			default:
				throw new SimpleGallery_Exception('Image format not supported: ' . $mimeType);
                break;
		}
    }

    /**
     * Check if image format is supported
     *
     * @throws SimpleGallery_Exception
     * @return void
     */
    protected function _checkFormatSupport()
    {
		$gdInfo	= gd_info();
        $formatSupported = true;

        switch ($this->_format) {
            case self::FORMAT_GIF:
                if (!$gdInfo['GIF Create Support']) {
                    $formatSupported = false;
                }
                break;

            case self::FORMAT_JPG:
                if (!$gdInfo['JPEG Support'] && !$gdInfo['JPG Support']) {
                    $formatSupported = false;
                }
                break;

            case self::FORMAT_PNG:
                if (!$gdInfo['PNG Support']) {
                    $formatSupported = false;
                }
                break;

            default:
                $formatSupported = false;
                break;
        }

        if (!$formatSupported) {
            throw new SimpleGallery_Exception('Image format not supported: ' . $this->_format);
        }

        // check if true color functions are available
        if (!function_exists('imagecreatetruecolor')) {
            $this->_trueColor = false;
        }
    }

    /**
     * Load image to internal GD resource
     *
     * @throws SimpleGallery_Exception
     * @return void
     */
    protected function _loadImage()
    {
        switch ($this->_format) {
			case self::FORMAT_GIF:
				$this->_resource = imagecreatefromgif($this->_file);
				break;

			case self::FORMAT_JPG:
				$this->_resource = imagecreatefromjpeg($this->_file);
				break;

			case self::FORMAT_PNG:
				$this->_resource = imagecreatefrompng($this->_file);
				break;

            default:
				throw new SimpleGallery_Exception('Image format not supported: ' . $this->_format);
                break;
		}
    }

    /**
     * Detect image dimensions
     *
     * @return void
     */
    protected function _detectDimensions()
    {
        $this->_width = imagesx($this->_resource);
        $this->_height = imagesy($this->_resource);

        $this->_ratio = $this->_width / $this->_height;
    }

    /**
     * Set options
     *
     * @param array $options Options to set
     * @throws SimpleGallery_Exception
     * @return void
     */
    protected function _setOptions(Array $options)
    {
        if (isset($options[self::OPTION_JPEG_QUALITY])) {
            if (!is_numeric($options[self::OPTION_JPEG_QUALITY])) {
                throw new SimpleGallery_Exception('Jpeg quality option must be numeric.');
            }

            $this->_options[self::OPTION_JPEG_QUALITY] = $options[self::OPTION_JPEG_QUALITY];
        }

        if (isset($options[self::OPTION_PRESERVE_RATIO])) {
            if (!is_bool($options[self::OPTION_PRESERVE_RATIO])) {
                throw new SimpleGallery_Exception('Preserve ratio option must be boolean.');
            }

            $this->_options[self::OPTION_PRESERVE_RATIO] = $options[self::OPTION_PRESERVE_RATIO];
        }
    }

    /**
     * Compute image size
     *
     * @param integer $maxWidth Maximum width
     * @param integer $maxHeight Maximum height
     * @return array An array width the following keys: width, height
     */
    protected function _calcImageSize($maxWidth, $maxHeight)
    {
        $imgSize = array(
            'width' => $maxWidth,
            'height' => $maxHeight
        );

        if (!$this->_options[self::OPTION_PRESERVE_RATIO]) {
            return $imgSize;
        }

        if ($this->_height < $this->_width) {
            $width = $maxWidth;
            $height = floor($this->_height * ($maxWidth / $this->_width) );
        } else {
            $height = $maxHeight;
            $width = floor($this->_width * ($maxHeight / $this->_height) );
        }

        $imgSize = array(
            'width' => $width,
            'height' => $height
        );

        return $imgSize;
    }

    /**
     * Compute strict image size - the resulting image will have one of
     * the dimensions the given value, the other will be larger. This
     * function is useful for cropping.
     *
     * @param integer $maxWidth Maximum width
     * @param integer $maxHeight Maximum height
     * @return array An array width the following keys: width, height
     */
    protected function _calcImageSizeStrict($maxWidth, $maxHeight)
    {
        $imgSize = array(
            'width' => $maxWidth,
            'height' => $maxHeight
        );

        if (!$this->_options[self::OPTION_PRESERVE_RATIO]) {
            return $imgSize;
        }

        if ($this->_height > $this->_width) {
            $width = $maxWidth;
            $height = floor($this->_height * ($maxWidth / $this->_width) );
        } else {
            $height = $maxHeight;
            $width = floor($this->_width * ($maxHeight / $this->_height) );
        }

        $imgSize = array(
            'width' => $width,
            'height' => $height
        );

        return $imgSize;
    }

    /**
     * Compute adaptive image size
     *
     * @param integer $maxWidth Maximum width
     * @param integer $maxHeight Maximum height
     * @return array An array width the following keys: width, height, cropWidth, cropHeight
     */
    protected function _calcAdaptiveImageSize($maxWidth, $maxHeight)
    {
        $imgSize = array(
            'width' => $maxWidth,
            'height' => $maxHeight,
            'cropWidth' => $maxWidth,
            'cropHeight' => $maxHeight
        );

        $imgSizeStrict = $this->_calcImageSizeStrict($maxWidth, $maxHeight);

        $imgSize['width'] = $imgSizeStrict['width'];
        $imgSize['height'] = $imgSizeStrict['height'];

        return $imgSize;
    }

    /**
     * Validate format
     * 
     * @param string $format Format to validate
     * @throws SimpleGallery_Exception
     * @return void
     */
    protected function _validateFormat($format)
    {
        $formatSupported = false;

        switch ($format) {
            case self::FORMAT_GIF:
            case self::FORMAT_JPG:
            case self::FORMAT_PNG:
                $formatSupported = true;
                break;

            default:
                $formatSupported = false;
                break;
        }

        if (!$formatSupported) {
            throw new SimpleGallery_Exception('Image format not supported: ' . $format);
        }
    }
}

/* EOF */